""" Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import sys
import eventlet
import requests
import time
import logging
import os
import json

from django.apps import AppConfig
from django.conf import settings
from expression_builder.exp import prepare_exp_list
from auth_ogre.requests_auth import get_requests_auth
from sealab.utils import maybe_json_or_raise
from workflow.errors.error_constants import *
from workflow.rabbitmq_utils import RabbitMQPublisher
from django.core.cache import cache
from workflow.constants import RECOMMENDATION_PREFERENCE

logger = logging.getLogger(__name__)


class WorkflowConfig(AppConfig):
    name = 'workflow'
    verbose_name = "Workflow"

    def ready(self):
        try:

            try:
                if 'uwsgi' in sys.argv:
                    settings.EXPRESSIONS = prepare_exp_list()
            except Exception as e:
                logger.warning('Unable to populate expressions due to error: %s', str(e))

            if 'eventlet' in sys.argv:
                eventlet.monkey_patch()
            logger.info('Updating proxy settings for the environment')
            if 'uwsgi' in sys.argv or 'worker' in sys.argv or 'beat' in sys.argv:
                configure_proxy()
                settings.ACTION_LIMIT_ENABLED = is_action_count_restriction_enabled(settings.CRUD_HUB_URL)
                create_rmq_publisher_placeholder()
                configure_recommendation_settings()
            import workflow.signals
        except Exception as e:
            logger.exception(str(e))


def configure_proxy(system_settings=None):
    try:
        config = {}
        if not system_settings:
            collection = settings.CRUD_HUB_URL + '/api/3/proxy'
            method = 'GET'
            auth_info = {'auth_method': 'CS HMAC'}
            try:
                auth = get_requests_auth(auth_info, collection, method,
                                         settings.APPLIANCE_PUBLIC_KEY,
                                         settings.APPLIANCE_PRIVATE_KEY)

                response = requests.request(method, collection, auth=auth, verify=False)
                system_settings = maybe_json_or_raise(response)
            except Exception as e:
                logger.error('Error while fetching proxy details %s', str(e))
                system_settings = None

        if system_settings:
            for k, v in system_settings.get('proxies', {}).items():
                config.update({k: v.get('value', '')})

            NO_PROXY = ','.join(system_settings.get('noProxy', []))
            if NO_PROXY:
                logger.info('Setting env variable %s', 'NO_PROXY')
                os.environ['NO_PROXY'] = NO_PROXY

            for el in system_settings.get('OTHER', []):
                key = list(el.keys())[0]
                logger.info('Setting env variable %s', key)
                os.environ[key] = el.get(key)

            for k, v in config.items():
                if v and system_settings.get('proxies', {}).get(k, {}).get('is_enabled'):
                    logger.info('Setting env variable %s', k)
                    os.environ[k] = v
                else:
                    logger.info('Clearing env variable %s', k)
                    os.environ.pop(k, None)
    except Exception as e:
        logger.exception(cs_wf_31.format(str(e)))


def update_action_count_restriction_details(daily_action_limit):
    from workflow.config import config_manager
    from workflow import constants
    from datetime import datetime, timedelta
    if not daily_action_limit:
        daily_action_limit = -1
    logger.info("Daily action limit received is: %s", daily_action_limit)
    try:
        already_configured_action_limit = config_manager.get_section(constants.LICENSE_SECTION)
        license_details = {constants.DAILY_ACTION_LIMIT: daily_action_limit}
        if daily_action_limit == -1:
            # Reset action restriction counters
            license_details[constants.REMAINING_ACTIONS] = license_details[constants.RESET_TIME] = license_details[
                constants.LAST_UPDATE_TIME] = -1
        elif already_configured_action_limit[constants.DAILY_ACTION_LIMIT] == -1:
            license_details[constants.LAST_UPDATE_TIME] = time.time()
            # Add last update time to remaining actions to prevent tampering
            license_details[constants.REMAINING_ACTIONS] = daily_action_limit + int(
                license_details[constants.LAST_UPDATE_TIME])
            license_details[constants.RESET_TIME] = int(
                (datetime.now() + timedelta(days=1)).replace(hour=0, minute=0, second=0, microsecond=0).timestamp())
        elif already_configured_action_limit[constants.DAILY_ACTION_LIMIT] != daily_action_limit:
            license_details[constants.REMAINING_ACTIONS] = already_configured_action_limit[constants.REMAINING_ACTIONS] \
                                                           + (daily_action_limit - already_configured_action_limit[constants.DAILY_ACTION_LIMIT])
        config_manager.update_section(constants.LICENSE_SECTION, license_details)
    except Exception as error:
        logger.error("Update action count details failed")
        logger.exception(error)


def is_action_count_restriction_enabled(app_url, retry_count=50, retry_interval=5):
    try:
        response = requests.get(app_url + '/auth/license/?param=license_details', verify=False)
        license_details = maybe_json_or_raise(response)
        daily_action_limit = license_details.get('details').get('entitlements').get('daily_action_limit')
        logger.info("updating action count details")
        update_action_count_restriction_details(daily_action_limit)
        if daily_action_limit is not None and daily_action_limit > 0:
            logger.info('returning action is_action_count_restriction_enabled as True')
            return True
    except Exception as error:
        logger.error('Could not retrieve license details from DAS %s', str(error))
        if retry_count > 1:
            time.sleep(retry_interval)
            logger.error('Retrying...')
            is_action_count_restriction_enabled(app_url, retry_count-1)
        else:
            logger.warn('Could not fetch license details from DAS. All retries exhausted. Considering action count '
                        'restriction disabled')
    return False


def create_rmq_publisher_placeholder():
    settings.RMQ_PUBLISHER = RabbitMQPublisher()


def configure_recommendation_settings(system_settings=None):
    try:
        if not system_settings:
            collection = settings.CRUD_HUB_URL + '/api/3/system_settings/845c05cc-05b3-450e-9afb-df6b6e436321'
            method = 'GET'
            auth_info = {'auth_method': 'CS HMAC'}
            try:
                auth = get_requests_auth(auth_info, collection, method,
                                         settings.APPLIANCE_PUBLIC_KEY,
                                         settings.APPLIANCE_PRIVATE_KEY)

                response = requests.request(method, collection, auth=auth, verify=False)
                system_settings = maybe_json_or_raise(response)
            except Exception as e:
                logger.error('Error while fetching system settings %s', str(e))
                system_settings = None

        if system_settings:
            recommendation_preference = system_settings['publicValues'].get('recommendation', {}).get('strategy', 'falconBased')
            cache.set(RECOMMENDATION_PREFERENCE, recommendation_preference, None)
    except Exception as e:
        logger.exception(cs_wf_49.format(str(e)))
